# -*- coding: utf-8 -*-

'''
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
'''


import base64
import urlparse
import urllib
import hashlib
import re

from resources.lib.modules import cleantitle
from resources.lib.modules import client
from resources.lib.modules import directstream
from resources.lib.modules import trakt
from resources.lib.modules import pyaes


RES_4K = ['4k', 'hd4k', '4khd', 'uhd', 'ultrahd', 'ultra-hd', '2160', '2160p', '2160i', 'hd2160', '2160hd',
          '1716p', '1716i', 'hd1716', '1716hd', '2664p', '2664i', 'hd2664', '2664hd', '3112p',
          '3112i', 'hd3112', '3112hd', '2880p', '2880i', 'hd2880', '2880hd']
RES_2K = ['2k', 'hd2k', 'hd2k ', '2khd', '2khd ', '2048p', '2048i', 'hd2048', '2048hd', '1332p', '1332i',
          'hd1332', '1332hd', '1556p', '1556i', 'hd1556', '1556hd']
RES_1080 = ['1080', '1080p', '1080i', 'hd1080', '1080hd', '1200p', '1200i', 'hd1200', '1200hd']
RES_HD = ['720', '720p', '720i', 'hd720', '720hd', '.hd.']
RES_SD = ['576', '576p', '576i', 'sd576', '576sd', '480', '480p', '480i', 'sd480', '480sd', '360', '360p',
          '360i', 'sd360', '360sd', '240', '240p', '240i', 'sd240', '240sd']

SCR = ['dvdscr', 'screener', 'r5', 'r6']

CAM = ['camrip', 'cam rip', 'tsrip', 'ts rip', 'dvdcam', 'dvd cam', 'dvdts', 'dvd ts', 'cam', 'telesync', 'tele sync',
       'ts']
HDCAM = ['hdcam', 'hd cam', 'hd-cam', 'hd.cam', 'hdts', 'hd ts', 'hcam']

CODEC_H265 = ['hevc', 'h265', 'h.265', 'x265', 'x.265', 'hdr']
CODEC_H264 = ['avc', 'h264', 'h.264', 'x264', 'x.264']
CODEC_XVID = ['xvid', 'x.vid', 'x-vid']
CODEC_DIVX = ['divx', 'divx ', 'div2', 'div2 ', 'div3']
CODEC_MPEG = ['mp4', 'mpeg', 'm4v', 'mpg', 'mpg1', 'mpg2', 'mpg3', 'mpg4', 'mp4 ', 'mpeg ', 'msmpeg', 'msmpeg4',
              'mpegurl']
CODEC_AVI = ['avi']
CODEC_MKV = ['mkv', '.mkv', 'matroska']

AUDIO_8CH = ['ch8', '8ch', 'ch7-1', '7-1ch', '7-1']
AUDIO_7CH = ['ch7', '7ch', 'ch6-1', '6-1ch', '6-1']
AUDIO_6CH = ['ch6', '6ch', 'ch5-1', '5-1ch', '5-1']
AUDIO_2CH = ['ch2', '2ch', 'ch2.0', 'stereo']
AUDIO_1CH = ['ch1', '1ch', 'mono', 'monoaudio']

VIDEO_3D = ['3d', 'sbs', 'hsbs', 'sidebyside', 'side by side', 'stereoscopic', 'tab', 'htab', 'topandbottom',
            'top and bottom']

MULTI_LANG = ['hindi.eng', 'ara.eng', 'ces.eng', 'chi.eng', 'cze.eng', 'dan.eng', 'dut.eng', 'ell.eng', 'esl.eng',
              'esp.eng', 'fin.eng', 'fra.eng', 'fre.eng', 'frn.eng', 'gai.eng', 'ger.eng', 'gle.eng', 'gre.eng',
              'gtm.eng', 'heb.eng', 'hin.eng', 'hun.eng', 'ind.eng', 'iri.eng', 'ita.eng', 'jap.eng', 'jpn.eng', 'kor.eng',
              'lat.eng', 'lebb.eng', 'lit.eng', 'nor.eng', 'pol.eng', 'por.eng', 'rus.eng', 'som.eng', 'spa.eng', 'sve.eng',
              'swe.eng', 'tha.eng', 'tur.eng', 'uae.eng', 'ukr.eng', 'vie.eng', 'zho.eng', 'dual audio', 'dual-audio',
              'dual.audio']

LANG = ['arabic', 'dutch', 'finnish', 'french', 'german', 'greek', 'italian', 'polish', 'portuguese', 'spanish',
              'truefrech', 'hebrew']

UNDESIREABLES = ['coldfilm', 'lostfilm', 'newstudio', 'vostfr']

DUBBED = ['dublado', 'dubbed']

SUBS = ['subs', 'subtitula']

ADDS = ['1xbet']


def get_qual(term):
    if any(i in term for i in RES_4K):
        return '4k'
    elif any(i in term for i in RES_1080):
        return '1080p'
    elif any(i in term for i in RES_720):
        return '720p'
    elif any(i in term for i in RES_SD):
        return 'sd'
    elif any(i in term for i in SCR):
        return 'scr'
    elif any(i in term for i in CAM):
        return 'cam'

def is_anime(content, type, type_id):
    try:
        r = trakt.getGenre(content, type, type_id)
        return 'anime' in r or 'animation' in r
    except:
        return False


def streaminfo(item, meta, type):
    info = {}
    try:
        if type == 'video':
            if item['quality'] == '4K':
                width = 3840
                height = 2160

            elif item['quality'] == '1080p':
                width = 1920
                height = 1080

            elif item['quality'] == '720p':
                width = 1280
                height = 720

            else:
                width = 720; height = 480

            try:
                if any(i in item['url'].lower() for i in CODEC_H265) or item['quality'] == '4K':
                    codec = 'hevc'
                elif any(i in item['url'].lower() for i in CODEC_H264):
                    codec = 'h264'
                elif any(i in item['url'].lower() for i in CODEC_XVID):
                    codec = 'xvid'
                else:
                    codec = 'h264'
            except:
                codec = 'h264'

            info = {'codec': codec, 'height': height, 'width': width, 'duration': meta['duration']}
            return info
        elif type == 'audio':
            if '5.1' in item['url'].lower() and any(x in item['url'].lower() for x in ['.dd.', 'dolby.digital', 'ac3']):
                codec = 'ac3'
                channels = 6

            elif '7.1' in item['url'].lower() and 'dtshd' in item['url'].lower():
                codec = 'dtshd'
                channels = 8
            elif '7.1' in item['url'].lower() and 'dolby' in item['url'].lower():
                codec = 'truehd'
                channels = 8

            else:
                codec = 'aac'; channels = 2

            if channels == 2:
                if '5.1' in item['url'].lower():
                    channels = 6
                elif '7.1' in item['url'].lower():
                    channels = 8

            if codec == 'aac':
                if '.dd' in item['url'].lower():
                    codec = 'dolbydigital'
                elif 'dolby' in item['url'].lower() and 'digital' in item['url'].lower():
                    codec = 'dolbydigital'
                elif '.ac3' in item['url'].lower():
                    codec = 'ac3'
                elif '.dts.' in item['url'].lower():
                    codec = 'dts'
                elif '.dtshd.' in item['url'].lower():
                    codec = 'dts'
                elif '.truehd.' in item['url'].lower():
                    codec = 'truehd'

            try:
                language = item['language']
            except:
                language = 'en'

            info = {'codec': codec, 'channels': channels, 'language': language}
            return info
    except:
        return info


def get_release_quality(release_name, release_link=None):

    if release_name is None:
        return

    try:
        release_name = release_name.encode('utf-8')
    except Exception:
        pass

    try:
        quality = None

        fmt = re.sub('[^A-Za-z0-9]+', ' ', release_name)
        fmt = str(fmt.lower())
        try:
            fmt = fmt.encode('utf-8')
        except Exception:
            pass

        if ' 2160p ' in fmt:
            quality = '4K'
        elif ' 2160 ' in fmt:
            quality = '4K'
        elif ' uhd ' in fmt:
            quality = '4K'
        elif ' 4k ' in fmt:
            quality = '4K'
        elif ' 1080p ' in fmt:
            quality = '1080p'
        elif ' 1080 ' in fmt:
            quality = '1080p'
        elif ' fullhd ' in fmt:
            quality = '1080p'
        elif ' 718p ' in fmt:
            quality = '720p'
        elif ' 720p ' in fmt:
            quality = '720p'
        elif ' hd ' in fmt:
            quality = '720p'
        elif ' 480p ' in fmt:
            quality = 'SD'
        elif ' 480 ' in fmt:
            quality = 'SD'
        elif ' 576p ' in fmt:
            quality = 'SD'
        elif ' 576 ' in fmt:
            quality = 'SD'
        elif any(i in [' dvdscr ', ' r5 ', ' r6 '] for i in fmt):
            quality = 'SCR'
        elif any(i in [' camrip ', ' tsrip ', ' hdcam ', ' hdts ', ' dvdcam ', ' dvdts ', ' cam ', ' telesync ', ' ts '] for i in fmt):
            quality = 'CAM'

        if not quality:
            if release_link:
                release_link = client.replaceHTMLCodes(release_link)
                release_link = re.sub('[^A-Za-z0-9 ]+', ' ', release_link)
                release_link = str(release_link.lower())

                try:
                    release_link = release_link.encode('utf-8')
                except Exception:
                    pass
                if ' 4k ' in release_link:
                    quality = '4k'
                elif ' 2160 ' in release_link:
                    quality = '4k'
                elif ' 2160p ' in release_link:
                    quality = '4k'
                elif ' uhd ' in release_link:
                    quality = '4k'
                elif ' 1080 ' in release_link:
                    quality = '1080p'
                elif ' 1080p ' in release_link:
                    quality = '1080p'
                elif ' 718p ' in release_link:
                    quality = '720p'
                elif ' fullhd ' in release_link:
                    quality = '1080p'
                elif ' 720p ' in release_link:
                    quality = '720p'
                elif ' hd ' in release_link:
                    quality = '720p'
                else:
                    if any(i in [' dvdscr ', ' r5 ', ' r6 '] for i in release_link):
                        quality = 'SCR'
                    elif any(i in [' camrip ', ' tsrip ', ' hdcam ', ' hdts ', ' dvdcam ', ' dvdts ', ' cam ', ' telesync ', ' ts '] for i in release_link):
                        quality = 'CAM'
                    else:
                        quality = 'SD'
            else:
                quality = 'SD'
        info = []
        if '3d' in fmt or '.3D.' in release_name:
            info.append('3D')
        if any(i in ['hevc', 'h265', 'x265'] for i in fmt):
            info.append('HEVC')

        return quality, info
    except Exception:
        return 'SD', []


def get_release_quality_crew(release_name, release_link=None):

    if release_name is None:
        return

    try:
        release_name = release_name.encode('utf-8')
    except:
        pass

    try:
        quality = None
        
        release_name = release_name.upper()

        fmt = re.sub('(.+)(\.|\(|\[|\s)(\d{4}|S\d*E\d*|S\d*)(\.|\)|\]|\s)', '', release_name)
        fmt = re.split('\.|\(|\)|\[|\]|\s|-', fmt)
        fmt = [i.lower() for i in fmt]
        if '4K' in fmt:
            quality = '4K'
        elif '2160p' in fmt:
            quality = '4K'
        elif '1080p' in fmt:
            quality = '1080p'
        elif '1080' in fmt:
            quality = '1080p'
        elif '720p' in fmt:
            quality = '720p'
        elif '720' in fmt:
            quality = '720p'
        elif 'brrip' in fmt:
            quality = '720p'
        elif 'hd' in fmt:
            quality = '720p'
        elif 'HD' in fmt:
            quality = '720p'
        elif '.hd.' in fmt:
            quality = '720p'
        elif 'hdtv' in fmt:
            quality = '720p'
        elif 'BluRay' in fmt:
            quality = '720p'
        elif '.WEBRip.' in fmt:
            quality = '720p'
        elif '480p' in fmt:
            quality = 'SD'
        elif '480' in fmt:
            quality = 'SD'
        elif any(i in ['dvdscr', 'r5', 'r6'] for i in fmt):
            quality = 'SCR'
        elif any(i in ['camrip', 'tsrip', 'hdcam', 'hdts', 'dvdcam', 'dvdts', 'cam', 'telesync', 'ts'] for i in fmt):
            quality = 'CAM'

        if not quality:
            if release_link:
                release_link = release_link.lower()
                try:
                    release_link = release_link.encode('utf-8')
                except:
                    pass
                if '4K' in release_link:
                    quality = '4K'
                elif '2160p' in release_link:
                    quality = '4K'
                elif '1080' in release_link:
                    quality = '1080p'
                elif '720' in release_link:
                    quality = '720p'
                elif '.hd' in release_link:
                    quality = '720p'
                else: 
                    if any(i in ['dvdscr', 'r5', 'r6'] for i in release_link):
                        quality = 'SCR'
                    elif any(i in ['camrip', 'tsrip', 'hdcam', 'hdts', 'dvdcam', 'dvdts', 'cam', 'telesync', 'ts'] for i in release_link):
                        quality = 'CAM'
                    else:
                        quality = 'SD'
            else:
                quality = 'SD'
        info = []
        if '3d' in fmt or '.3D.' in release_name:
            info.append('3D')
        if any(i in ['hevc', 'h265', 'x265'] for i in fmt):
            info.append('HEVC')

        return quality, info
    except:
        return 'SD', []


def getFileType(url):

    try:
        url = client.replaceHTMLCodes(url)
        url = re.sub('[^A-Za-z0-9]+', ' ', url)
        url = url.encode('utf-8')
        url = str(url.lower())
    except Exception:
        url = str(url)
    type = ''

    if any(i in url for i in [' bluray ', ' blu ray ', ' b lu ry ']):
        type += ' BLURAY /'
    if any(i in url for i in [' bd r ', ' bdr ', ' bd rip ', ' bdrip ', ' br rip ', ' brrip ']):
        type += ' BD-RIP /'
    if ' remux ' in url:
        type += ' REMUX /'
    if any(i in url for i in [' dvdrip ', ' dvd rip ']):
        type += ' DVD-RIP /'
    if any(i in url for i in [' dvd ', ' dvdr ', ' dvd r ']):
        type += ' DVD /'
    if any(i in url for i in [' webdl ', ' web dl ', ' web ', ' web rip ', ' webrip ']):
        type += ' WEB /'
    if ' hdtv ' in url:
        type += ' HDTV /'
    if ' sdtv ' in url:
        type += ' SDTV /'
    if any(i in url for i in [' hdrip ', ' hd rip ']):
        type += ' HDRIP /'
    if any(i in url for i in [' uhdrip ', ' uhd rip ']):
        type += ' UHDRIP /'
    if ' r5 ' in url:
        type += ' R5 /'
    if any(i in url for i in [' cam ', ' cam rip ', ' camrip ']):
        type += ' CAM /'
    if any(i in url for i in [' ts ', ' telesync ', ' hdts ', ' pdvd ']):
        type += ' TS /'
    if any(i in url for i in [' tc ', ' telecine ', ' hdtc ']):
        type += ' TC /'
    if any(i in url for i in [' scr ', ' screener ', ' dvdscr ', ' dvd scr ']):
        type += ' SCR /'
    if ' xvid ' in url:
        type += ' XVID /'
    if ' avi ' in url:
        type += ' AVI /'
    if any(i in url for i in [' h 264 ', ' h264 ', ' x264 ', ' avc ']):
        type += ' H.264 /'
    if any(i in url for i in [' h 265 ', ' h256 ', ' x265 ', ' hevc ']):
        type += ' HEVC /'
    if ' hi10p ' in url:
        type += ' HI10P /'
    if ' 10bit ' in url:
        type += ' 10BIT /'
    if ' 3d ' in url:
        type += ' 3D /'
    if ' hdr ' in url:
        type += ' HDR /'
    if ' imax ' in url:
        type += ' IMAX /'
    if ' ac3 ' in url:
        type += ' AC3 /'
    if ' eac3 ' in url:
        type += ' EAC3 /'
    if ' aac ' in url:
        type += ' AAC /'
    if any(i in url for i in [' dd ', ' dolby ', ' dolbydigital ', ' dolby digital ']):
        type += ' DD /'
    if any(i in url for i in [' truehd ', ' true hd ']):
        type += ' TRUEHD /'
    if ' atmos ' in url:
        type += ' ATMOS /'
    if any(i in url for i in [' ddplus ', ' dd plus ', ' ddp ']):
        type += ' DD+ /'
    if ' dts ' in url:
        type += ' DTS /'
    if any(i in url for i in [' hdma ', ' hd ma ']):
        type += ' HD.MA /'
    if any(i in url for i in [' hdhra ', ' hd hra ']):
        type += ' HD.HRA /'
    if any(i in url for i in [' dtsx ', ' dts x ']):
        type += ' DTS:X /'
    if ' dd5 1 ' in url:
        type += ' DD / 5.1 /'
    if any(i in url for i in [' 5 1 ', ' 6ch ']):
        type += ' 5.1 /'
    if any(i in url for i in [' 7 1 ', ' 8ch ']):
        type += ' 7.1 /'
    if 'subs' in url:
        if type != '':
            type += ' - WITH SUBS'
        else:
            type = 'SUBS'
    type = type.rstrip('/')
    return type


def check_sd_url(release_link):
    try:
        release_link = re.sub('[^A-Za-z0-9]+', ' ', release_link)
        release_link = str(release_link.lower())
        try:
            release_link = release_link.encode('utf-8')
        except Exception:
            pass
        if ' 2160 ' in release_link:
            quality = '4K'
        elif ' 2160p ' in release_link:
            quality = '4K'
        elif ' 4k ' in release_link:
            quality = '4K'
        elif ' uhd ' in release_link:
            quality = '4K'
        elif ' 1080 ' in release_link:
            quality = '1080p'
        elif ' 1080p ' in release_link:
            quality = '1080p'
        elif ' fullhd ' in release_link:
            quality = '1080p'
        elif ' 720p ' in release_link:
            quality = '720p'
        elif ' hd ' in release_link:
            quality = '720p'
        elif any(i in [' dvdscr ', ' r5 ', ' r6 '] for i in release_link):
            quality = 'SCR'
        elif any(i in [' camrip ', ' tsrip ', ' hdcam ', ' hdts ', ' dvdcam ', ' dvdts ', ' cam ', ' telesync ', ' ts '] for i in release_link):
            quality = 'CAM'
        else:
            quality = 'SD'
        return quality
    except Exception:
        return 'SD'


def check_direct_url(url):
    try:
        url = url.lower()
        if '4k' in url:
            quality = '4K'
        elif '2160p' in url:
            quality = '4K'
        elif '2160' in url:
            quality = '4K'
        elif 'uhd' in url:
            quality = '4K'
        elif '1080p' in url:
            quality = '1080p'
        elif '1080' in url:
            quality = '1080p'
        elif '720p' in url:
            quality = '720p'
        elif '720' in url:
            quality = '720p'
        elif any(i in ['dvdscr', 'r5', 'r6'] for i in url):
            quality = 'SCR'
        elif any(i in ['camrip', 'tsrip', 'hdcam', 'hdts', 'dvdcam', 'dvdts', 'cam', 'telesync', 'ts'] for i in url):
            quality = 'CAM'
        else:
            quality = 'SD'
        return quality
    except:
        return 'SD'


def check_all_url(url):
    try:
        url = url.lower()
        if '4k' in url:
            quality = '4K'
        elif '2160p' in url:
            quality = '4K'
        elif '2160' in url:
            quality = '4K'
        elif 'uhd' in url:
            quality = '4K'
        elif '1080p' in url:
            quality = '1080p'
        elif '1080' in url:
            quality = '1080p'
        elif '720p' in url:
            quality = '720p'
        elif '720' in url:
            quality = '720p'
        elif any(i in ['dvdscr', 'r5', 'r6'] for i in url):
            quality = 'SCR'
        elif any(i in ['camrip', 'tsrip', 'hdcam', 'hdts', 'dvdcam', 'dvdts', 'cam', 'telesync', 'ts'] for i in url):
            quality = 'CAM'
        elif 'fembed' in url:
            quality = '720p'
        elif 'clicknupload' in url:
            quality = '720p'
        else:
            quality = 'SD'
        return quality
    except:
        return 'SD'


def check_url(url):

    try:
        url = url.lower()
        if '4k' in url: quality = '4K'
        elif '2160' in url: quality = '4K'
        elif '1080' in url: quality = '1080p'
        elif '720' in url: quality = '720p'
        elif '.hd.' in url: quality = '720p'
        elif '480p' in url: quality = 'SD'
        elif '480' in url: quality = 'SD'
        elif any(i in ['dvdscr', 'r5', 'r6'] for i in url): quality = 'SCR'
        elif any(i in ['camrip', 'tsrip', 'hdcam', 'hdts', 'dvdcam', 'dvdts', 'cam', 'telesync', 'ts'] for i in url): quality = 'CAM'
        else: quality = 'SD'
        return quality
    except:
        return 'SD'


def check_title(title, name, hdlr, year):
    # from openscrapers.modules import log_utils
    try:
        match = True
        n = name.lower()
        h = hdlr.lower()
        t = n.split(h)[0].replace(year, '').replace('(', '').replace(')', '').replace('&', 'and').replace('.us.', '.')
        # log_utils.log('cleantitle.get(t) = %s' % cleantitle.get(t), log_utils.LOGDEBUG)
        # log_utils.log('cleantitle.get(title) = %s' % cleantitle.get(title), log_utils.LOGDEBUG)
        if cleantitle.get(t) != cleantitle.get(title):
            match = False
        if h not in n:
            match = False
        return match
    except:
        match = False


def label_to_quality(label):
    try:
        try: label = int(re.search('(\d+)', label).group(1))
        except: label = 0

        if label >= 2160:
            return '4K'
        elif label >= 1440:
            return '1440p'
        elif label >= 1080:
            return '1080p'
        elif 720 <= label < 1080:
            return '720p'
        elif label < 720:
            return 'SD'
    except:
        return 'SD'


def strip_domain(url):
    try:
        if url.lower().startswith('http') or url.startswith('/'):
            url = re.findall('(?://.+?|)(/.+)', url)[0]
        url = client.replaceHTMLCodes(url)
        url = url.encode('utf-8')
        return url
    except:
        return


def is_host_valid(url, domains):
    try:
        if any(x in url.lower() for x in ['.rar.', '.zip.', '.iso.']):
            raise Exception()
        if any(url.lower().endswith(x) for x in ['.rar', '.zip', '.iso']):
            raise Exception()
        if any(x in url.lower() for x in ['youtube', 'sample', 'trailer', 'zippyshare', 'facebook']):
            raise Exception()

        host = __top_domain(url)
        hosts = [domain.lower() for domain in domains if host and host in domain.lower()]

        if hosts and '.' not in host:
            host = hosts[0]
        if hosts and any([h for h in ['google', 'picasa', 'blogspot'] if h in host]):
            host = 'gvideo'
        if hosts and any([h for h in ['akamaized', 'ocloud'] if h in host]):
            host = 'CDN'
        return any(hosts), host
    except Exception:
        return False, ''


def __top_domain(url):
    elements = urlparse.urlparse(url)
    domain = elements.netloc or elements.path
    domain = domain.split('@')[-1].split(':')[0]
    regex = "(?:www\.)?([\w\-]*\.[\w\-]{2,3}(?:\.[\w\-]{2,3})?)$"
    res = re.search(regex, domain)
    if res: domain = res.group(1)
    domain = domain.lower()
    return domain


def aliases_to_array(aliases, filter=None):
    try:
        if not filter:
            filter = []
        if isinstance(filter, str):
            filter = [filter]

        return [x.get('title') for x in aliases if not filter or x.get('country') in filter]
    except:
        return []


def append_headers(headers):
    return '|%s' % '&'.join(['%s=%s' % (key, urllib.quote_plus(headers[key])) for key in headers])


def get_size(url):
    try:
        size = client.request(url, output='file_size')
        if size == '0': size = False
        size = convert_size(size)
        return size
    except: return False


def convert_size(size_bytes):
   import math
   if size_bytes == 0:
       return "0B"
   size_name = ("B", "KB", "MB", "GB", "TB", "PB", "EB", "ZB", "YB")
   i = int(math.floor(math.log(size_bytes, 1024)))
   p = math.pow(1024, i)
   s = round(size_bytes / p, 2)
   if size_name[i] == 'B' or size_name[i] == 'KB': return None
   return "%s %s" % (s, size_name[i])


def check_directstreams(url, hoster='', quality='SD'):
    urls = []
    host = hoster

    if 'google' in url or any(x in url for x in ['youtube.', 'docid=']):
        urls = directstream.google(url)
        if not urls:
            tag = directstream.googletag(url)
            if tag: urls = [{'quality': tag[0]['quality'], 'url': url}]
        if urls: host = 'gvideo'
    elif 'ok.ru' in url:
        urls = directstream.odnoklassniki(url)
        if urls: host = 'vk'
    elif 'vk.com' in url:
        urls = directstream.vk(url)
        if urls: host = 'vk'
    elif any(x in url for x in ['akamaized', 'blogspot', 'ocloud.stream']):
        urls = [{'url': url}]
        if urls: host = 'CDN'
        
    direct = True if urls else False

    if not urls: urls = [{'quality': quality, 'url': url}]

    return urls, host, direct


# if salt is provided, it should be string
# ciphertext is base64 and passphrase is string
def evp_decode(cipher_text, passphrase, salt=None):
    cipher_text = base64.b64decode(cipher_text)
    if not salt:
        salt = cipher_text[8:16]
        cipher_text = cipher_text[16:]
    data = evpKDF(passphrase, salt)
    decrypter = pyaes.Decrypter(pyaes.AESModeOfOperationCBC(data['key'], data['iv']))
    plain_text = decrypter.feed(cipher_text)
    plain_text += decrypter.feed()
    return plain_text


def evpKDF(passwd, salt, key_size=8, iv_size=4, iterations=1, hash_algorithm="md5"):
    target_key_size = key_size + iv_size
    derived_bytes = ""
    number_of_derived_words = 0
    block = None
    hasher = hashlib.new(hash_algorithm)
    while number_of_derived_words < target_key_size:
        if block is not None:
            hasher.update(block)

        hasher.update(passwd)
        hasher.update(salt)
        block = hasher.digest()
        hasher = hashlib.new(hash_algorithm)

        for _i in range(1, iterations):
            hasher.update(block)
            block = hasher.digest()
            hasher = hashlib.new(hash_algorithm)

        derived_bytes += block[0: min(len(block), (target_key_size - number_of_derived_words) * 4)]

        number_of_derived_words += len(block) / 4

    return {
        "key": derived_bytes[0: key_size * 4],
        "iv": derived_bytes[key_size * 4:]
    }

def remove_lang(name):
    try:
        name = name.lower()
        name = name.replace(' ', '.')
    except:
        name = str(name)

    if any(value in name for value in LANG):
        return True
    elif any(value in name for value in UNDESIREABLES):
        return True
    elif any(value in name for value in DUBBED):
        return True
    elif 'rus' in name and 'eng' not in name.lower():
        return True
    else:
        return False

def _size(siz):
    if siz in ['0', 0, '', None]: return 0, ''
    div = 1 if siz.lower().endswith(('gb', 'gib')) else 1024
    float_size = float(re.sub('[^0-9|/.|/,]', '', siz.replace(',', '.'))) / div
    str_size = '%.2f GB' % float_size
    return float_size, str_size

